searchNodes=[{"doc":"Rationale Profiling Elixir code is easy using the default Erlang tools, such as fprof . These tools produce a lot of potentially useful data, but visualizing and interpreting all that data is not easy. The erlang tool eflame contains some utilities to generate flamegraphs from the results of profiling your code But... eflame expects you to manually run a bash script, which according to my reading seems to call a perl (!) script that generates an interactive SVG. And although the generated SVGs support some minimal interaction, it's possible to do better. When developping a web application, one can take advantage of the web browser as a highly dynamic tool for visualization using SVG, HTML and Javascript. Fortunately there is a very good javascrtip library to generate flamegraphs: d3-flame-graph . By reading :eflame stacktrace samples and converting them into a format that d3-flame-graph can understand, we can render the flamegraph in a webpage. That way, instead of the manual steps above you can just visit an URL in your web application. Usage To use incendium in your web application, you need to follow these steps: 1. Add Incendium as a dependency def deps do [ { :incendium , &quot;~&gt; 0.1.0&quot; } ] end You can make it a :dev only dependency if you wish, but Incendium will only decorate your functions if you're in :dev mode. Incendium decorators won't decorate your functions in :prod (profilers such as eflame should never be used in :prod because they add a very significant overhead; your code will be ~10-12 times slower) 2. Create an Incendium Controller for your application # lib/my_app_web/controllers/incencdium_controller.ex defmodule MyApp.IncendiumController do use Incendium.Controller , routes_module : MyAppWeb.Router.Helpers , otp_app : :my_app end There is no need to define an accompanying view. Currently the controller is not extensible (and there aren't many natural extension points anyway). Upon compilation, the controller will automcatically add the files incendium.js and incendium.css to your priv/static directory so that those static files will be served using the normal Phoenix mechanisms. On unusual Phoenix apps which have static files in other places, this might not work as expected. Currently there isn't a way to override the place where the static files should be added. 3. Add the controller to your Router # lib/my_app_web/controllers/router.ex require Incendium scope &quot;/incendium&quot; , MyAppWeb do Incendium . routes ( IncendiumController ) end 4. Decorate the functions you want to profile Incendium decorators depend on the decorator package. defmodule MyAppWeb.MyContext.ExampleController do use MyAppWeb.Mandarin , :controller # Activate the incendium decorators use Incendium.Decorator # Each invocation of the `index/2` function will be traced and profiled. @decorate incendium_profile_with_tracing ( ) def index ( conn , params ) do resources = MyContext . list_resources ( params ) render ( conn , &quot;index.html&quot; , resources : resources ) end end Currently incendium only supports tracing profilers (which are very slow and not practical in production). In the future we may support better options such as sampling profilers. 5. Visit the /incendium route to see the generated flamegraph Each time you run a profiled function, a new stacktrace will be generated. Stacktraces are not currently saves, you can only access the latest one. In the future we might add a persistence layer that stores a number of stacktraces instead of keeping just the last one.","ref":"Incendium.html","title":"Incendium","type":"module"},{"doc":"Profiles a function call using tracing (as opposed to sampling). This profiler uses :eflame under the hood. Profiled code will be very slow. Never use this in production. Stack data will be written to incendium/stacks.out.","ref":"Incendium.html#profile_with_tracing/1","title":"Incendium.profile_with_tracing/1","type":"function"},{"doc":"Adds routes for an Incendium controller.","ref":"Incendium.html#routes/1","title":"Incendium.routes/1","type":"macro"},{"doc":"","ref":"Incendium.Controller.html","title":"Incendium.Controller","type":"module"},{"doc":"Turn the current module into an Icendium controller. Options: routes_module (required) - The module that contains the routes in your application (it should be something like YourApp.Router.Helpers ) :otp_app (required) - The main OTP application Examples defmodule MyApp.IncendiumController do use Incendium.Controller , routes_module : MyApp.Router.Helpers , otp_app : :my_app end","ref":"Incendium.Controller.html#__using__/1","title":"Incendium.Controller.__using__/1","type":"macro"},{"doc":"Defines decorators to profile function calls.","ref":"Incendium.Decorator.html","title":"Incendium.Decorator","type":"module"},{"doc":"","ref":"Incendium.Decorator.html#incendium_profile_with_tracing/0","title":"Incendium.Decorator.incendium_profile_with_tracing/0","type":"macro"},{"doc":"Decorator . When the decorated function is invoked, it will bd profiled using tracing. Stack data will be written to incendium/stacks.out. Note : profiling will be disabled if Mix.env() == :prod . This means it's safe(ish) to leave these decorators in a production environment. It's never a good idea to profile in prod using the :eflame library, as it makes your code run ~10x slower. Example defmodule MyApp.UserController do # ... use Incendium.Decorator @decorate incendium_profile_with_tracing ( ) def index ( conn , params ) do # ... end end","ref":"Incendium.Decorator.html#incendium_profile_with_tracing/2","title":"Incendium.Decorator.incendium_profile_with_tracing/2","type":"function"},{"doc":"","ref":"Mix.Incendium.Assets.html","title":"Mix.Incendium.Assets","type":"module"},{"doc":"Example flamegraph This is an example flamegraph generated by Incendium. Flamegraphs are highly dynamic thanks to the d3 library. Ýou can click on a horizontal rectangle (which represents a function call) to zoom in on the rectangle and its descendants. You can use the search form on the top right to search for a specific function name. The Download link allows you to download the flamegraph as a javascript file. To draw the flamegraph in a webpage you just have to add an inline &lt;script&gt; that references the javascript file. You don't even need to call any javascript. The script automatically inserts the flamgeraph just before the script tag. For the flamegraph to work properly, you must include the incendium.js and incendium.css in the webpage (the generated javascript code dependes on the javascript in incendium.js ). While incendium could be changed in order to generate completely standalone javascript, if you want to include several flamegrpahs in the same webpage, including the entire contents of incendium.js would be very wasteful. For example, the flamegraph above was generated by the following HTML: &lt;link rel=&quot;stylesheet&quot; href=&quot;assets/incendium.css&quot; /&gt; &lt;script src=&quot;assets/incendium.js&quot; charset=&quot;utf-8&quot;&gt;&lt;/script&gt; &lt;script type=&quot;text/javascript&quot; src=&quot;assets/incendium_flamegraph_hkctthqlqhcubcsgrazymmvaldzllxbq.js&quot;&gt;&lt;/script&gt;","ref":"example-flamegraph.html","title":"Example flamegraph","type":"extras"}]